/*******************************************************************/
/* reduction de la palette de couleur d'une bitmap 256 couleurs    */
/* par U-Foot                                                      */
/*******************************************************************/

/*******************************************************************/
/* redcol8.c                                                       */
/*******************************************************************/

/*=================================================================*/
/* includes                                                        */
/*=================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <allegro.h>
#include <sys/stat.h>

/*=================================================================*/
/* variables globales                                              */
/*=================================================================*/

int   ARGC;
char  **ARGV;
int   FLAG_HELP=0;
int   FLAG_SILENT=0;
char  *FILENAMES[65536];
int   NUMBER_OF_FILES=0;
PALETTE PALETTE_SRC;
BITMAP *BITMAP_SRC=NULL;
BITMAP *BITMAP_DST=NULL;
signed char  BUFFER[1048576];
int   DST_X,DST_Y,DST_W,DST_H,DST_SIZE,BG_SIZE;

/*=================================================================*/
/* fonctions                                                       */
/*=================================================================*/

/*-----------------------------------------------------------------*/
/* affichages d'aide                                               */
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
void display_short_help (void)
{
 printf ("mktab 1.0 by U-Foot.\n");
 printf ("Creates maps for LW5.\n");
 printf ("It uses Allegro 3.0 by Shawn Hargreaves.\n");
 printf ("Type 'mktab -? for more help.\n");
}

/*-----------------------------------------------------------------*/
void display_long_help (void)
{
 printf ("mktab 1.0 by U-Foot.\n");
 printf ("Creates maps for LW5, insert the created files in lw.dat.\n");
 printf ("The source must be a 256 color bitmap.\n");
 printf ("Color 0=walls, the map must be contained in a closed rectangle.\n");
 printf ("Other colors ignored.\n");
 printf ("It uses Allegro 3.0 by Shawn Hargreaves.\n");
 printf ("Syntax:\n");
 printf ("mktab filename1 filename2 etc...\n");
}

/*-----------------------------------------------------------------*/
/* lecture des parametres de la ligne de commande                  */
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
int acknowledge_flag (char *str)
{
 int found=0;

 if (str[0]=='-' || str[0]=='/')
    {
     found=1;
     switch (str[1])
     {
      case '?':
      case 'h':
      case 'H': FLAG_HELP=1;
                break;
      case 's':
      case 'S': FLAG_SILENT=1;
                break;
      default : found=0;
     }
    }
 return found;
}

/*-----------------------------------------------------------------*/
int read_command_line (void)
{
 int      i,j=0,k,success=1;

 for (i=1; i<ARGC; ++i)
     {
      if ((!acknowledge_flag (ARGV[i])))
         {
             FILENAMES[j]=ARGV[i];
          j++;
         }
     }
 NUMBER_OF_FILES=j;
 if (NUMBER_OF_FILES<=0)
     {
      success=0;
      if (!FLAG_HELP)
         printf ("ERROR! Two few arguments.\n");
     }
 return success;
}

/*-----------------------------------------------------------------*/
/* acces disque                                                    */
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
int load_file (char *filename)
{
 int success;

 if (!FLAG_SILENT)
    printf ("Loading '%s'.\n", filename);

 if (BITMAP_SRC != NULL)
    destroy_bitmap (BITMAP_SRC);
 BITMAP_SRC=load_bitmap (filename, PALETTE_SRC);
 success=BITMAP_SRC!=NULL;
 if (!success)
    printf ("ERROR! Unable to read '%s'.\n", filename);
 return success;
}

/*-----------------------------------------------------------------*/
/* lecture des parametres du tableau                               */
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
void get_range (void)
{
 int min_x=BITMAP_SRC->w;
 int min_y=BITMAP_SRC->h;
 int max_x=-1;
 int max_y=-1;
 int x,y;

 for (y=0;y<BITMAP_SRC->h;++y)
     for (x=0;x<BITMAP_SRC->w;++x)
         if (!getpixel (BITMAP_SRC,x,y))
            {
             if (min_x>x)
                min_x=x;
             if (min_y>y)
                min_y=y;
             if (max_x<x)
                max_x=x;
             if (max_y<y)
                max_y=y;
            }
 DST_X=min_x;
 DST_Y=min_y;
 DST_W=max_x-min_x+1;
 DST_H=max_y-min_y+1;

 if (DST_W<1 || DST_H<1)
    {
     printf ("ERROR! Map is too small.\n");
     DST_W=DST_H=1;
    }
 if (BITMAP_DST!=NULL)
    destroy_bitmap (BITMAP_DST);
 BITMAP_DST=create_bitmap (DST_W,DST_H);
 blit (BITMAP_SRC, BITMAP_DST, DST_X, DST_Y, 0, 0, DST_W, DST_H);
}

/*-----------------------------------------------------------------*/
/* conversion dans le buffer                                       */
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
void convert_to_buffer (void)
{
 int pos_src,pos_dst,l,wh;
 char *data;

 wh=BITMAP_DST->h*BITMAP_DST->w;
 data=BITMAP_DST->dat;
 pos_src=pos_dst=0;
 BG_SIZE=0;

 while (pos_src<wh)
   {
   l=0;
   if (!data[pos_src])
      {
       while ((!data[pos_src]) && pos_src<wh && l<127)
         {
          l++;
          pos_src++;
         }
       BUFFER[pos_dst++]=l;
      }
   else
      {
       while (data[pos_src] && pos_src<wh && l<127)
         {
          l++;
          pos_src++;
         }
       BUFFER[pos_dst++]=-l;
       BG_SIZE+=l;
      }
   }
 BUFFER[pos_dst++]=0;

 DST_SIZE=pos_dst;
}

/*-----------------------------------------------------------------*/
/* on ecrit sur le disque                                          */
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
void change_ext (char *str)
{
  char *ext;

  ext=strchr (str, '.');
  ext[1]='m';
  ext[2]='a';
  ext[3]='p';
}

/*-----------------------------------------------------------------*/
void write_to_disk (char *str)
{
 int f;
 short w,h;
 int size;

 printf ("Writing %s to disk.\n",str);
 delete_file (str);
 f=open (str,O_TRUNC|O_CREAT|O_BINARY|O_APPEND, S_IWUSR);
 w=DST_W;
 h=DST_H;
 size=DST_SIZE;
 write (f,&BG_SIZE,4);
 write (f,&w,2);
 write (f,&h,2);
 write (f,BUFFER,DST_SIZE);
 close (f);
}

/*-----------------------------------------------------------------*/
void write_with_new_size (char *str)
{
 int f;
 short w,h;
 int size;

 printf ("Writing %s to disk.\n",str);
 delete_file (str);

 save_bitmap (str,BITMAP_DST,PALETTE_SRC);
}

/*=================================================================*/
/* fonctions main                                                  */
/*=================================================================*/

/*-----------------------------------------------------------------*/
int main (int argc, char **argv)
{
 int  i;

 ARGC=argc;
 ARGV=argv;

 allegro_init ();

 if (read_command_line ())
    {
     for (i=0; i<NUMBER_OF_FILES; ++i)
         {
          if (load_file (FILENAMES [i]))
             {
              get_range ();
              write_with_new_size(FILENAMES [i]);
              convert_to_buffer ();
              change_ext (FILENAMES [i]);
              write_to_disk (FILENAMES [i]);
              }
        }
    }
 else
     {
      if (FLAG_HELP)
         display_long_help ();
      else
          display_short_help ();
     }
 allegro_exit ();

 return 0;

}

